@@ -24,6 +24,13 @@ use nat::*;
 /// Models a type relation `a : t`.
 pub type Ty<A, T> = And<Imply<A, T>, POrdProof<A, T>>;
 
+/// `x : ltrue`.
+pub fn ty_ltrue<X: LProp>() -> Ty<X, LTrue<S<X::N>>>
+    where X::N: Lt<S<X::N>> + Default
+{
+    (LTrue(Default::default()).map_any(), POrdProof::default())
+}
+
 /// `(a : b) ⋀ (a == c)  =>  (c : b)`.
 pub fn ty_in_left_arg<A: Prop, B: Prop, C: Prop>((ab, pord): Ty<A, B>, eq: Eq<A, C>) -> Ty<C, B> {
     (imply::in_left_arg(ab, eq.clone()), pord.by_eq_left(eq))
@@ -34,6 +41,28 @@ pub fn ty_in_right_arg<A: Prop, B: Prop, C: Prop>((ab, pord): Ty<A, B>, eq: Eq<B
     (imply::in_right_arg(ab, eq.clone()), pord.by_eq_right(eq))
 }
 
+/// `true == ltrue`.
+pub fn eq_true_ltrue<N: Nat>() -> Eq<True, LTrue<N>> {
+    (LTrue(Default::default()).map_any(), True.map_any())
+}
+
+/// `x : true`.
+pub fn ty_true<X: LProp>() -> Ty<X, True>
+    where X::N: Nat
+{
+    ty_in_right_arg(ty_ltrue(), eq::symmetry(eq_true_ltrue::<S<X::N>>()))
+}
+
+/// `true ~~ x`.
+pub fn q_true<X: LProp>() -> Q<True, X> where X::N: Nat {
+    univalence::h0_q_true(ty_true())
+}
+
+/// `x ~~ x`.
+pub fn true_q<X: LProp>() -> Q<X, X> where X::N: Nat {
+    univalence::h0_true_q(ty_true())
+}
+
 /// `(x : a) ⋀ (y : b)  =>  ((x ⋀ y) : (a ⋀ b))`.
 pub fn ty_and<X: Prop, Y: Prop, A: Prop, B: Prop>(
     (xa, pord_xa): Ty<X, A>,
@@ -102,6 +131,52 @@ pub fn ty_eqq_imply<X: DProp, Y: DProp, A: Prop, B: Prop>(
     (xy_ab, pord)
 }
 
+/// `x`.
+pub fn lproof<X: LProp>() -> X where X::N: Nat {
+    let q_true_x = q_true();
+    quality::to_eq(q_true_x).0(True)
+}
+
+/// `(x ~~ x) == x`.
+pub fn lim<X: LProp>() -> Eq<Q<X, X>, X> where X::N: Nat {
+    univalence::h0_lim(ty_true())
+}
+
+/// `(x ~~ x) ~~ x`.
+pub fn qlim<X: LProp>() -> Q<Q<X, X>, X> where X::N: Nat {
+    univalence::h0_qlim(ty_true())
+}
+
+/// `(a : (b ⋁ c))  =>  (a : b) ⋁ (a : c)`.
+pub fn ty_or_split_da<A: DProp, B: Prop, C: Prop>(
+    (ty_a_or_b_c, pord): Ty<A, Or<B, C>>
+) -> Or<Ty<A, B>, Ty<A, C>> {
+    match imply::or_split_da(ty_a_or_b_c) {
+        Left(ty_a_b) => Left((ty_a_b, pord.or_left())),
+        Right(ty_a_c) => Right((ty_a_c, pord.or_right()))
+    }
+}
+
+/// `(a : (b ⋁ c))  =>  (a : b) ⋁ (a : c)`.
+pub fn ty_or_split_db<A: Prop, B: DProp, C: Prop>(
+    (ty_a_or_b_c, pord): Ty<A, Or<B, C>>
+) -> Or<Ty<A, B>, Ty<A, C>> {
+    match imply::or_split_db(ty_a_or_b_c) {
+        Left(ty_a_b) => Left((ty_a_b, pord.or_left())),
+        Right(ty_a_c) => Right((ty_a_c, pord.or_right()))
+    }
+}
+
+/// `(a : (b ⋁ c))  =>  (a : b) ⋁ (a : c)`.
+pub fn ty_or_split_dc<A: Prop, B: Prop, C: DProp>(
+    (ty_a_or_b_c, pord): Ty<A, Or<B, C>>
+) -> Or<Ty<A, B>, Ty<A, C>> {
+    match imply::or_split_dc(ty_a_or_b_c) {
+        Left(ty_a_b) => Left((ty_a_b, pord.or_left())),
+        Right(ty_a_c) => Right((ty_a_c, pord.or_right()))
+    }
+}
+
 /// Core axiom of Path Semantics.
 pub type PSem<F1, F2, X1, X2> = Imply<
     And<And<Q<F1, F2>, And<POrdProof<F1, X1>, POrdProof<F2, X2>>>,
@@ -217,6 +292,18 @@ impl<T, U> POrdProof<T, U> {
     }
 }
 
+impl<A, B, C> POrdProof<A, Or<B, C>> {
+    /// Get sub order proof `a < b`.
+    pub fn or_left(self) -> POrdProof<A, B> {
+        POrdProof(std::marker::PhantomData)
+    }
+
+    /// Get sub order proof `a < c`.
+    pub fn or_right(self) -> POrdProof<A, C> {
+        POrdProof(std::marker::PhantomData)
+    }
+}
+
 /// Path semantical order.
 ///
 /// This is implemented by types to define an order
@@ -251,6 +338,10 @@ impl LProp for False {
     type N = NaN;
     type SetLevel<T: 'static + Clone> = Self;
 }
+impl<U: LProp> LProp for Q<U, U> {
+    type N = U::N;
+    type SetLevel<T: 'static + Clone> = Q<U::SetLevel<T>, U::SetLevel<T>>;
+}
 /// Increases proposition level of `A` with some amount `N`.
 pub type IncLevel<A, N> = <A as LProp>::SetLevel<<(<A as LProp>::N, N) as Add>::Out>;
 /// Increases level one step.
